@@ -75,6 +75,7 @@ extern void GPUreadDataMem(uint32_t *, int);
 extern long GPUdmaChain(uint32_t *,uint32_t);
 extern void GPUupdateLace(void);
 extern long GPUfreeze(uint32_t, void *);
+extern void GPUvBlank(int);
 
 
 #define DUMMY(id, name) \
@@ -166,6 +167,7 @@ static const struct {
 	DIRECT_GPU(GPUwriteDataMem),
 	DIRECT_GPU(GPUdmaChain),
 	DIRECT_GPU(GPUfreeze),
+	DIRECT_GPU(GPUvBlank),
 
 	DUMMY_GPU(GPUdisplayText),
 /*
@@ -178,7 +180,6 @@ static const struct {
 	DIRECT_GPU(GPUshowScreenPic),
 */
 //	DIRECT_GPU(GPUclearDynarec),
-//	DIRECT_GPU(GPUvBlank),
 };
 
 void *plugin_link(enum builtint_plugins_e id, const char *sym)
